@@ -4,15 +4,14 @@ use strict;
 use warnings;
 use Moo;
 use Protocol::WebSocket::Frame;
-use Scalar::Util qw( weaken );
+use Scalar::Util ();
 use Encode ();
 use AE;
-use AnyEvent;
 use AnyEvent::WebSocket::Message;
-use Carp qw( croak carp );
+use Carp ();
 
 # ABSTRACT: WebSocket connection for AnyEvent
-our $VERSION = '0.33'; # VERSION
+our $VERSION = '0.34'; # VERSION
 
 
 has handle => (
@@ -53,7 +52,7 @@ has "_is_finished" => (
 sub BUILD
 {
   my $self = shift;
-  weaken $self;
+  Scalar::Util::weaken $self;
   my $finish = sub {
     my $strong_self = $self; # preserve $self because otherwise $self can be destroyed in the callbacks.
     return if $self->_is_finished;
@@ -165,7 +164,7 @@ sub on
   }
   else
   {
-    croak "unrecongized event: $event";
+    Carp::croak "unrecongized event: $event";
   }
   $self;
 }
@@ -173,9 +172,16 @@ sub on
 
 sub close
 {
-  my($self) = @_;
+  my($self, $code, $reason) = @_;
 
-  $self->send(AnyEvent::WebSocket::Message->new(opcode => 8, body => ""));
+  my $body = pack('n', ($code) ? $code : '1005');
+
+  $body .= Encode::encode 'UTF-8', $reason if defined $reason;
+
+  $self->send(AnyEvent::WebSocket::Message->new(
+    opcode => 8,
+    body => $body,
+  ));
   $self->handle->push_shutdown;
   $self->_is_write_open(0);
   $self;
@@ -195,7 +201,7 @@ AnyEvent::WebSocket::Connection - WebSocket connection for AnyEvent
 
 =head1 VERSION
 
-version 0.33
+version 0.34
 
 =head1 SYNOPSIS
 
@@ -292,13 +298,18 @@ Called only for the next message received from the WebSocket.
 
 Called when the connection is terminated
 
-=head3
-
 =head2 close
 
  $connection->close;
+ $connection->close($code);
+ $connection->close($code, $reason);
 
-Close the connection.
+Close the connection.  You may optionally provide a code and a reason.
+See L<section 5.5.1|https://tools.ietf.org/html/rfc6455#section-5.5.1> and L<section 7.4.1|https://tools.ietf.org/html/rfc6455#section-7.4.1> of RFC6455.
+
+The code is a 16-bit unsigned integer value that indicates why you close the connection. By default the code is 1005.
+
+The reason is a character string (not an octet string) that further describes why you close the connection. By default the reason is an empty string.
 
 =head1 SERVER CONNECTIONS
 
@@ -334,6 +345,8 @@ L<RFC 6455 The WebSocket Protocol|http://tools.ietf.org/html/rfc6455>
 
 =back
 
+=for stopwords Joaquín José
+
 =head1 AUTHOR
 
 Author: Graham Ollis E<lt>plicease@cpan.orgE<gt>
@@ -342,6 +355,8 @@ Contributors:
 
 Toshio Ito
 
+José Joaquín Atria
+
 =head1 COPYRIGHT AND LICENSE
 
 This software is copyright (c) 2013 by Graham Ollis.
